==> process_run-0.11.2-dev.2.tar.gz: lib/src/lines_utils.dart <==
import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'common/import.dart';

/// Basic shell lines controller.
///
/// Usage:
/// ```dart
/// var controller = ShellLinesController();
/// var shell = Shell(stdout: controller.sink, verbose: false);
/// controller.stream.listen((event) {
///   // Handle output
///
///   // ...
///
///   // If needed kill the shell
///   shell.kill();
/// });
/// try {
///   await shell.run('dart echo.dart some_text');
/// } on ShellException catch (_) {
///   // We might get a shell exception
/// }
/// ```
class ShellLinesController {
  final _controller = StreamController<List<int>>();

  /// The sink for the Shell stderr/stdout
  StreamSink<List<int>> get sink => _controller.sink;

  /// The stram to listen to
  Stream<String> get stream => shellStreamLines(_controller.stream);

  /// Dispose the controller.
  void close() {
    _controller.close();
  }
}

/// Basic line streaming. Assuming system encoding
Stream<String> shellStreamLines(Stream<List<int>> stream,
    {Encoding encoding = systemEncoding}) {
  StreamSubscription subscription;
  List<int> currentLine;
  const endOfLine = 10;
  const lineFeed = 13;
  StreamController<String> ctlr;
  encoding ??= systemEncoding;
  ctlr = StreamController<String>(onListen: () {
    void addCurrentLine() {
      if (currentLine?.isNotEmpty ?? false) {
        try {
          ctlr.add(systemEncoding.decode(currentLine));
        } catch (_) {
// Ignore nad encoded line
          print('ignoring: $currentLine');
        }
      }
      currentLine = null;
    }

    void addToCurrentLine(List<int> data) {
      if (currentLine == null) {
        currentLine = data;
      } else {
        var newCurrentLine = Uint8List(currentLine.length + data.length);
        newCurrentLine.setAll(0, currentLine);
        newCurrentLine.setAll(currentLine.length, data);
        currentLine = newCurrentLine;
      }
    }

    subscription = stream.listen((data) {
      // var _w;
      // print('read $data');
      // devPrint('read $data');
      // look for \n (10)
      var start = 0;
      for (var i = 0; i < data.length; i++) {
        var byte = data[i];
        if (byte == endOfLine || byte == lineFeed) {
          addToCurrentLine(data.sublist(start, i));
          addCurrentLine();
// Skip it
          start = i + 1;
        }
      }
      // Store last current line
      if (data.length > start) {
        addToCurrentLine(data.sublist(start, data.length));
      }
    }, onDone: () {
      // Last one
      if (currentLine != null) {
        addCurrentLine();
      }
      ctlr.close();
    });
  }, onCancel: () {
    subscription?.cancel();
  });

  return ctlr.stream;
}

==> process_run-0.12.0.tar.gz: lib/src/lines_utils.dart <==
import 'dart:convert';
import 'dart:io';
import 'dart:typed_data';

import 'common/import.dart';

/// Basic shell lines controller.
///
/// Usage:
/// ```dart
/// var controller = ShellLinesController();
/// var shell = Shell(stdout: controller.sink, verbose: false);
/// controller.stream.listen((event) {
///   // Handle output
///
///   // ...
///
///   // If needed kill the shell
///   shell.kill();
/// });
/// try {
///   await shell.run('dart echo.dart some_text');
/// } on ShellException catch (_) {
///   // We might get a shell exception
/// }
/// ```
class ShellLinesController {
  final _controller = StreamController<List<int>>();

  /// The sink for the Shell stderr/stdout
  StreamSink<List<int>> get sink => _controller.sink;

  /// The stram to listen to
  Stream<String> get stream => shellStreamLines(_controller.stream);

  /// Dispose the controller.
  void close() {
    _controller.close();
  }
}

/// Basic line streaming. Assuming system encoding
Stream<String> shellStreamLines(Stream<List<int>> stream,
    {Encoding encoding = systemEncoding}) {
  StreamSubscription? subscription;
  List<int>? currentLine;
  const endOfLine = 10;
  const lineFeed = 13;
  late StreamController<String> ctlr;
  ctlr = StreamController<String>(onListen: () {
    void addCurrentLine() {
      if (currentLine?.isNotEmpty ?? false) {
        try {
          ctlr.add(systemEncoding.decode(currentLine!));
        } catch (_) {
// Ignore nad encoded line
          print('ignoring: $currentLine');
        }
      }
      currentLine = null;
    }

    void addToCurrentLine(List<int> data) {
      if (currentLine == null) {
        currentLine = data;
      } else {
        var newCurrentLine = Uint8List(currentLine!.length + data.length);
        newCurrentLine.setAll(0, currentLine!);
        newCurrentLine.setAll(currentLine!.length, data);
        currentLine = newCurrentLine;
      }
    }

    subscription = stream.listen((data) {
      // var _w;
      // print('read $data');
      // devPrint('read $data');
      // look for \n (10)
      var start = 0;
      for (var i = 0; i < data.length; i++) {
        var byte = data[i];
        if (byte == endOfLine || byte == lineFeed) {
          addToCurrentLine(data.sublist(start, i));
          addCurrentLine();
// Skip it
          start = i + 1;
        }
      }
      // Store last current line
      if (data.length > start) {
        addToCurrentLine(data.sublist(start, data.length));
      }
    }, onDone: () {
      // Last one
      if (currentLine != null) {
        addCurrentLine();
      }
      ctlr.close();
    });
  }, onCancel: () {
    subscription?.cancel();
  });

  return ctlr.stream;
}
